@@ -1,43 +1,69 @@
-densityMclust <- function(data, ...) 
+densityMclust <- function(data, ..., plot = TRUE) 
 {
   mc <- match.call()
   obj <- Mclust(data, ...)
+  if(is.null(obj)) return(obj)
   obj$call <- mc
-  d <- dens(modelName = obj$modelName, data = data, 
-            parameters = obj$parameters, logarithm = FALSE)
-  obj$density <- d
+  obj$density <- dens(data = obj$data, 
+                      modelName = obj$modelName,
+                      parameters = obj$parameters, 
+                      logarithm = FALSE)
   class(obj) <- c("densityMclust", "Mclust")
+  if(plot) plot(obj, what = "density")
   return(obj)
 }
 
-predict.densityMclust <- function(object, newdata, what = c("dens", "cdens"), logarithm = FALSE, ...)
+predict.densityMclust <- function(object, newdata, 
+	                                what = c("dens", "cdens", "z"), 
+																	logarithm = FALSE, ...)
 {
   if(!inherits(object, "densityMclust")) 
-    stop("object not of class \"densityMclust\"")
+    stop("object not of class 'densityMclust'")
   if(missing(newdata))
     { newdata <- object$data }
   newdata <- as.matrix(newdata)
   if(ncol(object$data) != ncol(newdata))
     { stop("newdata must match ncol of object data") }
+  what <- match.arg(what, choices = eval(formals(predict.densityMclust)$what))	
+  pro <- object$parameters$pro; pro <- pro/sum(pro)
+  noise <- (!is.na(object$hypvol))
+  cl <- c(seq(object$G), if(noise) 0)
 
-  what <- match.arg(what)
-  if(what == "dens")
-  { 
-    d <- dens(modelName = object$modelName, 
-              data = newdata, 
-              parameters = object$parameters,
-              logarithm = logarithm) 
-  } else
-  { 
-    d <- cdens(modelName = object$modelName, 
-               data = newdata, 
-               parameters = object$parameters,
-               logarithm = logarithm)
-      dim <- dim(d)
-      attributes(d) <- NULL 
-      d <- array(d, dim)
-  }
-  return(d)
+  switch(what,
+         "dens" = 
+				 {  
+					 out <- dens(data = newdata, 
+					             modelName = object$modelName, 
+                       parameters = object$parameters,
+                       logarithm = logarithm) 
+         },
+				 "cdens" =
+         {
+					 z <- cdens(data = newdata, 
+					            modelName = object$modelName, 
+					            parameters = object$parameters,
+					            logarithm = TRUE)
+					 z <- if(noise) cbind(z, log(object$parameters$Vinv))
+                else      cbind(z) # drop redundant attributes
+           colnames(z) <- cl
+           out <- if(!logarithm) exp(z) else z
+         },
+				 "z" = 
+				 {
+					 z <- cdens(data = newdata, 
+					            modelName = object$modelName, 
+                      parameters = object$parameters,
+                      logarithm = TRUE)
+					 z <- if(noise) cbind(z, log(object$parameters$Vinv))
+                else      cbind(z) # drop redundant attributes
+           z <- sweep(z, MARGIN = 2, FUN = "+", STATS = log(pro))
+           z <- sweep(z, MARGIN = 1, FUN = "-", STATS = apply(z, 1, logsumexp))
+           colnames(z) <- cl
+           out <- if(!logarithm) exp(z) else z
+         }
+	)
+	
+  return(out)
 }
 
 plot.densityMclust <- function(x, data = NULL, what = c("BIC", "density", "diagnostic"), ...) 
@@ -89,11 +115,11 @@ plot.densityMclust <- function(x, data = NULL, what = c("BIC", "density", "diagn
 }
 
 
-plotDensityMclust1 <- function(x, data = NULL, hist.col = "lightgrey", hist.border = "white", breaks = "Sturges", ...) 
+plotDensityMclust1 <- function(x, data = NULL, col = gray(0.3), hist.col = "lightgrey", hist.border = "white", breaks = "Sturges", ...) 
 {
   object <- x # Argh.  Really want to use object anyway
   mc <- match.call(expand.dots = TRUE)
-  mc$x <- mc$data <- mc$hist.col <- mc$hist.border <- mc$breaks <- NULL
+  mc$x <- mc$data <- mc$col <- mc$hist.col <- mc$hist.border <- mc$breaks <- NULL
   xlab <- mc$xlab
   if(is.null(xlab)) 
     xlab <- deparse(object$call$data)
@@ -111,10 +137,10 @@ plotDensityMclust1 <- function(x, data = NULL, hist.col = "lightgrey", hist.bord
   d <- predict.densityMclust(object, eval.points)
   #
   if(!is.null(data)) 
-  { h <- hist(data, breaks = breaks, plot = FALSE)
+  { 
+    h <- hist(data, breaks = breaks, plot = FALSE)
     plot(h, freq = FALSE, col = hist.col, border = hist.border, main = "",
          xlim = range(h$breaks, xrange), 
-         # ylim = range(0, ylim, h$density, max(d)+diff(range(d))*0.1),
          ylim =  if(!is.null(ylim)) range(ylim) else range(0, h$density, d),
          xlab = xlab, ylab = ylab)
     box()
@@ -122,13 +148,15 @@ plotDensityMclust1 <- function(x, data = NULL, hist.col = "lightgrey", hist.bord
     mc$x <- eval.points
     mc$y <- d
     mc$type <- "l"
+    mc$col <- col
     eval(mc, parent.frame())
-  }
-  else
-  { mc[[1]] <- as.name("plot")
+  } else
+  { 
+    mc[[1]] <- as.name("plot")
     mc$x <- eval.points
     mc$y <- d
     mc$type <- "l"
+    mc$col <- col
     mc$xlim <- xlim
     mc$ylim <- if(!is.null(ylim)) range(ylim) else range(0, d)
     mc$ylab <- ylab
@@ -172,7 +200,7 @@ plotDensityMclust2 <- function(x, data = NULL,
   par <- object$parameters
   # these parameters should be missing 
   par$variance$cholSigma <- par$Sigma <- NULL
-  if(is.null(par$pro)) par$pro <- 1  # LS: bug?
+  if(is.null(par$pro)) par$pro <- 1
   par$variance$cholsigma <- par$variance$sigma
   for(k in seq(par$variance$G))
      { par$variance$cholsigma[,,k] <- chol(par$variance$sigma[,,k]) }
@@ -217,8 +245,8 @@ plotDensityMclustd <- function(x, data = NULL,
   
   nc <- object$d
   oldpar <- par(mfrow = c(nc, nc), 
-                mar = rep(c(gap,gap/2),each=2), 
-                oma = c(4, 4, 4, 4),
+                mar = rep(gap/2,4), 
+                oma = rep(3, 4),
                 no.readonly = TRUE)
   on.exit(par(oldpar))
 
@@ -265,43 +293,42 @@ plotDensityMclustd <- function(x, data = NULL,
   invisible()
 }
 
-dens <- function(modelName, data, logarithm = FALSE, parameters, warn = NULL, ...)
+dens <- function(data, modelName, parameters, logarithm = FALSE, warn = NULL, ...)
 {
   if(is.null(warn)) warn <- mclust.options("warn")
   # aux <- list(...)
-  cden <- cdens(modelName = modelName, data = data,
-                logarithm = TRUE, parameters = parameters, warn = warn)
-  dimdat <- dim(data)
-  oneD <- is.null(dimdat) || NCOL(data) == 1
-  G <- if(oneD) { length(parameters$mean) }
-       else     { ncol(as.matrix(parameters$mean)) }
+  logcden <- cdens(data = data,
+                   modelName = modelName,
+                   parameters = parameters,
+                   logarithm = TRUE, 
+                   warn = warn)
   pro <- parameters$pro
   if(is.null(pro))
     stop("mixing proportions must be supplied")
   noise <- (!is.null(parameters$Vinv))
-  if(G > 1)
-    { if(noise) 
-        { # proN <- pro[length(pro)]
-          pro <- pro[-length(pro)]
-          # pro <- pro/sum(pro)
-      }
-      if(any(proz <- pro == 0)) 
-        { pro <- pro[!proz]
-          cden <- cden[, !proz, drop = FALSE]
-      }
-      cden <- sweep(cden, 2, FUN = "+", STATS = log(pro))
-  }
-  # logsumexp
-  maxlog <- apply(cden, 1, max)
-  cden <- sweep(cden, 1, FUN = "-", STATS = maxlog)
-  den <- log(apply(exp(cden), 1, sum)) + maxlog
   if(noise) 
-    den <- den + parameters$pro[G+1]*parameters$Vinv
-  if(!logarithm) den <- exp(den)
-  den
+  {
+    proNoise <- pro[length(pro)]
+    pro <- pro[-length(pro)]
+  }
+  if(any(proz <- pro == 0)) 
+  { 
+    pro <- pro[!proz]
+    logcden <- logcden[, !proz, drop = FALSE]
+  }
+  logcden <- sweep(logcden, 2, FUN = "+", STATS = log(pro))
+  # logsumexp
+  maxlog <- apply(logcden, 1, max)
+  logcden <- sweep(logcden, 1, FUN = "-", STATS = maxlog)
+  logden <- log(apply(exp(logcden), 1, sum)) + maxlog
+  #
+  if(noise) 
+    logden <- log(exp(logden) + proNoise*parameters$Vinv)
+  out <- if(logarithm) logden else exp(logden)
+  return(out)
 }
 
-cdens <- function(modelName, data, logarithm = FALSE, parameters, warn = NULL, ...)
+cdens <- function(data, modelName, parameters, logarithm = FALSE, warn = NULL, ...)
 {
   modelName <- switch(EXPR = modelName,
                       X = "E",
@@ -437,6 +464,7 @@ cdfMclust <- function(object, data, ngrid = 100, ...)
   return(out)
 }
 
+# old version using spline approx
 quantileMclust <- function(object, p, ...)
 {
 # Calculate the quantile of a univariate mixture corresponding to cdf equal to p 
@@ -447,13 +475,48 @@ quantileMclust <- function(object, p, ...)
 
   if(!any(class(object) == "densityMclust"))
     { stop("first argument must be an object of class 'densityMclust'") }
-  
   eval.points <- extendrange(object$data, f = 1)
   eval.points <- seq(eval.points[1], eval.points[2], length.out = 10000) 
   cdf <- cdfMclust(object, data = eval.points)
-  q <- spline(cdf$y, cdf$x, method = "fmm", xmin = 0, xmax = 1, xout = p)$y
+  dup <- duplicated(cdf$y)
+  q <- spline(cdf$y[!dup], cdf$x[!dup], method = "fmm", 
+              xmin = 0, xmax = 1, xout = p)$y
   q[ p < 0 | p > 1] <- NaN
   q[ p == 0 ] <- -Inf
   q[ p == 1 ] <- Inf
   return(q)  
-}
\ No newline at end of file
+}
+
+quantileMclust <- function(object, p, ...)
+{
+# Calculate the quantile of a univariate mixture corresponding to cdf
+# equal to p using bisection line search method.
+#
+# Arguments:
+# object = a 'densityMclust' object
+# p = vector of probabilities (0 <= p <= 1)
+
+  stopifnot(inherits(object, "densityMclust"))
+  if(object$d != 1)
+    { stop("quantile function only available for 1-dimensional data") }
+  p <- as.vector(p)
+  m <- object$parameters$mean
+  s <- sqrt(object$parameters$variance$sigmasq)
+  if(object$modelName == "E") s <- rep(s, object$G)
+  r <- matrix(as.double(NA), nrow = length(p), ncol = object$G)
+  for(g in 1:object$G)
+  {
+    r[,g] <- qnorm(p, mean = m[g], sd = s[g])
+  }
+  if(object$G == 1) return(as.vector(r))
+  q <- rep(as.double(NA), length(p))
+  for(i in 1:length(p))
+  {
+    F <- function(x) cdfMclust(object, x)$y - p[i]
+    q[i] <- uniroot(F, interval = range(r[i,]), tol = sqrt(.Machine$double.eps))$root
+  }
+  q[ p < 0 | p > 1] <- NaN
+  q[ p == 0 ] <- -Inf
+  q[ p == 1 ] <- Inf
+  return(q)  
+}
