@@ -173,7 +173,8 @@ mclust1Dplot <- function(data, parameters = NULL, z = NULL,
            if(is.null(parameters$pro) && parameters$variance$G != 1) 
              stop("mixing proportions missing")
            x <- grid1(n = ngrid, range = xlim, edge = TRUE)
-           plot(x, dens("V", data = x, parameters = parameters),
+           plot(x, dens(data = x, modelName = "V", 
+                        parameters = parameters),
                 xlab = xlab, 
                 ylab = if(is.null(ylab)) "Density" else ylab, 
                 xlim = xlim, 
@@ -494,7 +495,7 @@ mvn2plot <- function(mu, sigma, k = 15, alone = FALSE,
 }
 
 clPairs <- function (data, classification, 
-                     symbols = NULL, colors = NULL, cex = 1, 
+                     symbols = NULL, colors = NULL, cex = NULL, 
                      labels = dimnames(data)[[2]], cex.labels = 1.5, 
                      gap = 0.2, grid = FALSE, ...) 
 {
@@ -532,6 +533,7 @@ clPairs <- function (data, classification,
     { colors <- rep( "black", l)
       warning("more colors needed")
   }
+	if(is.null(cex)) cex <- rep(1, l) 
   grid <- isTRUE(as.logical(grid))
   
   if(d > 2)
@@ -542,12 +544,12 @@ clPairs <- function (data, classification,
             },
             pch = symbols[classification], 
             col = colors[classification], 
+            cex = cex[classification], 
             gap = gap, 
-            cex = cex, 
             cex.labels = cex.labels,
             ...) }
   else if(d == 2)
-    { plot(data, cex = cex, 
+    { plot(data, cex = cex[classification],
             pch = symbols[classification], 
             col = colors[classification], 
             panel.first = if(grid) grid(),
@@ -556,10 +558,11 @@ clPairs <- function (data, classification,
   invisible(list(d = d,
                  class = levels(classification),
                  col = colors,
-                 pch = symbols[seq(l)]))
+                 pch = symbols[seq(l)],
+								 cex = cex))
 }
 
-clPairsLegend <- function(x, y, class, col, pch, box = TRUE, ...)
+clPairsLegend <- function(x, y, class, col, pch, cex, box = TRUE, ...)
 {
   
   usr <- par("usr")
@@ -581,9 +584,10 @@ clPairsLegend <- function(x, y, class, col, pch, box = TRUE, ...)
   dots$legend <- class
   dots$text.width <- max(strwidth(dots$title, units = "user"), 
                          strwidth(dots$legend, units = "user"))
-  dots$col <- col
-  dots$text.col <- col
-  dots$pch <- pch
+  dots$col <- if(missing(col)) 1 else col
+  dots$text.col <- if(missing(col)) 1 else col
+  dots$pch <- if(missing(pch)) 1 else pch
+	dots$cex <- if(missing(cex)) 1 else cex
   dots$title.col <- par("fg")
   dots$title.adj <- 0.1
   dots$xpd <- NA
@@ -1094,7 +1098,7 @@ surfacePlot <- function(data, parameters,
                         transformation = c("none", "log", "sqrt"), 
                         grid = 200, nlevels = 11, levels = NULL, 
                         prob = c(0.25, 0.5, 0.75),
-                        col = gray(0.7),
+                        col = gray(0.5),
                         col.palette = function(...) hcl.colors(..., "blues", rev = TRUE),
                         hdr.palette = blue2grey.colors,
                         xlim = NULL, ylim = NULL, 
@@ -1253,7 +1257,7 @@ surfacePlot <- function(data, parameters,
            p3d <- do.call("persp", 
                           c(list(x = x, y = y, z = zz, border = NA,
                                  xlab = xlab, ylab = ylab, 
-                                 col = col,
+                                 col = adjustcolor(col, alpha.f = 0.5),
                                  zlab = "Density", main = ""), dots))
            ii <- floor(seq(1, length(y), length.out = 2*nlevels))
            for(i in ii[-c(1,length(ii))])
